/*****************************************************************************
 * Copyright (c) 2006 Rowley Associates Limited.                             *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

  .section .vectors, "ax"
  .code 32
  .align 0
  .global _vectors
  .global reset_handler

/* The symbols
   INTERNAL_SRAM_START_ADDRESS,     
   SDRAM_START_ADDRESS, SDRAM_LENGTH
   are defined externally either with #define or using linker symbols
*/

#define MMU_TRANSLATION_TABLE_START_ADDRESS SDRAM_START_ADDRESS
  
/*****************************************************************************
 *                                                                           *
 * Exception Vectors                                                         *
 *                                                                           *
 *****************************************************************************/
_vectors:
  ldr pc, [pc, #reset_handler_address - . - 8]
  ldr pc, [pc, #undef_handler_address - . - 8]
  ldr pc, [pc, #swi_handler_address - . - 8]
  ldr pc, [pc, #pabort_handler_address - . - 8]
  ldr pc, [pc, #dabort_handler_address - . - 8]
  nop
  ldr pc, [pc, #-0xF20]
  ldr pc, [pc, #fiq_handler_address - . - 8]

reset_handler_address:
  .word reset_handler
undef_handler_address:
  .word undef_handler
swi_handler_address:
  .word swi_handler
pabort_handler_address:
  .word pabort_handler
dabort_handler_address:
  .word dabort_handler
irq_handler_address:
  .word irq_handler
fiq_handler_address:
  .word fiq_handler

  .section .init, "ax"
  .code 32
  .align 0

/******************************************************************************
 *                                                                            *
 * Default exception handlers                                                 *
 *                                                                            *
 ******************************************************************************/
reset_handler:

  /* Assumes that a boot loader has run which has 
       
       - initialised the clocks/PLL
       - initialised the SDRAM
       - remapped Internal SRAM to address zero

     If you have a board that boots from EBI NCS0 you'll need to do the above steps */    

  /* Copy exception vectors into Internal SRAM */
  ldr r0, =INTERNAL_SRAM_START_ADDRESS
  ldr r1, =_vectors
  ldmia r1!, {r2-r9}
  stmia r0!, {r2-r9}
  ldmia r1!, {r2-r8}
  stmia r0!, {r2-r8}

#ifndef NO_CACHE_ENABLE
  /* Set the tranlation table base address */
  ldr r0, =MMU_TRANSLATION_TABLE_START_ADDRESS
  mcr p15, 0, r0, c2, c0, 0          /* Write to TTB register */

  /* Setup the domain access control so accesses are not checked */
  ldr r0, =0xFFFFFFFF
  mcr p15, 0, r0, c3, c0, 0          /* Write to domain access control register */

  /* Create translation table */
  ldr r0, =MMU_TRANSLATION_TABLE_START_ADDRESS
  bl libarm_mmu_flat_initialise_level_1_table

  /* Make the SDRAM cacheable */
  ldr r0, =MMU_TRANSLATION_TABLE_START_ADDRESS
  ldr r1, =SDRAM_START_ADDRESS 
  ldr r2, =SDRAM_LENGTH 
  bl libarm_mmu_flat_set_level_1_cacheable_region

  /* Enable the MMU and caches */
  mrc p15, 0, r0, c1, c0, 0          /* Read MMU control register */
  orr r0, r0, #0x00001000            /* Enable ICache */
//  orr r0, r0, #0x00000007            /* Enable DCache, MMU and alignment fault */  
  orr r0, r0, #0x00000003            /* Enable MMU and alignment fault */
  mcr p15, 0, r0, c1, c0, 0          /* Write MMU control register */
  nop
  nop
#endif

  /****************************************************************************
   * Jump to the default C runtime startup code.                              *
   ****************************************************************************/

  b _start

/******************************************************************************
 *                                                                            *
 * Default exception handlers                                                 *
 * These are declared weak symbols so they can be redefined in user code.     * 
 *                                                                            *
 ******************************************************************************/

undef_handler:
  b undef_handler
  
swi_handler:
  b swi_handler
  
pabort_handler:
  b pabort_handler
  
dabort_handler:
  b dabort_handler
  
irq_handler:
  b irq_handler
  
fiq_handler:
  b fiq_handler

  .weak undef_handler, swi_handler, pabort_handler, dabort_handler, irq_handler, fiq_handler

