/* Part of FreeRTOS porting code on Blackfin processor */
/* Luke Yang <magic.yyang@gmail.com> */

#include "FreeRTOSConfig.h"
#include "context_sl_asm.h"

.extern _vTaskSwitchContext;
.extern _configUSE_PREEMPTION;
.extern _vTaskIncrementTick;

.section program;

/*
 * Manual context switch.  The first thing we do is save the registers so we
 * can use a naked attribute.
 */
 /*  This function is also the ISR for interrupt 2 */
 .global _vContextSwitch;
_vContextSwitch:
    _portSAVE_CONTEXT;

	/* Call the standard scheduler context switch function. */
	LINK 0x0;
	SP += 12;
	call.X _vTaskSwitchContext;
	UNLINK;

    _portRESTORE_CONTEXT;
_vContextSwitch.end:


/* 
 * The ISR used for the scheduler tick depends on whether the cooperative or
 * the preemptive scheduler is being used.
 */
/* The cooperative scheduler requires a normal IRQ service routine to 
simply increment the system tick. */

/* This is the ISR for core timer interrupt (default level 6)*/
.global _vTickISR;
_vTickISR:

#if (configUSE_PREEMPTION == 0)

		LINK 0x0;
		SP += 12;
		call.X _vTaskIncrementTick;
		UNLINK;

#else

		/* Save the context of the interrupted task. */
		_portSAVE_CONTEXT;	

		/* Increment the RTOS tick count, then look for the highest priority 
		task that is ready to run. */
		LINK 0x0;
		SP += 12;
		call.X _vTaskIncrementTick;
		UNLINK;
		LINK 0x0;
		SP += 12;
		call.X _vTaskSwitchContext;
		UNLINK;
		
		/* Restore the context of the new task. */
		_portRESTORE_CONTEXT;
	
#endif
_vTickISR.end:
