/* 
	FreeRTOS.org V4.1.3 - Copyright (C) 2003-2006 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section 
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license 
	and contact details.  Please ensure to read the configuration and relevant 
	port sections of the online documentation.
	***************************************************************************
*/

#ifndef PORTMACRO_H
#define PORTMACRO_H

/*-----------------------------------------------------------
 * Port specific definitions.  
 *
 * The settings in this file configure FreeRTOS correctly for the
 * given hardware and compiler.
 *
 * These settings should not be altered.
 *-----------------------------------------------------------
 */
 
/* Type definitions. */
#define portCHAR		char
#define portFLOAT		float
#define portDOUBLE		double
#define portLONG		long
#define portSHORT		short
#define portSTACK_TYPE	unsigned portLONG
#define portBASE_TYPE	portLONG

#if( configUSE_16_BIT_TICKS == 1 )
	typedef unsigned portSHORT portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffff
#else
	typedef unsigned portLONG portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffffff
#endif
/*-----------------------------------------------------------*/	

/* Scheduler utilities. */

/* Critical section management. */
#define portDISABLE_INTERRUPTS()				\
	asm volatile ("CLI R0;	\n\t				\
				   [ -- SP ] = R0;	\n\t"		\
				   ::							\
				   : "R0"						\
				 )
			
#define portENABLE_INTERRUPTS()						\
	asm volatile (".extern _freertos_global_imask;\n\t	\
				   R0 = [ SP ++ ];	\n\t			\
				   /* if interrupt is not nest disabled */	\
				   CC = BITTST( R0, 6 ); \n\t		\
				   /* if nest disabled, just restore original IMASK */ \
				   if !CC JUMP 22;	\n\t			\
				   /* interrupt to be enabled, update IMASK from global_imask */		\
				   [ -- SP ] = R1;	\n\t			\
				   [ -- SP ] = P1;	\n\t			\
				   P1.L = _freertos_global_imask;\n\t	\
				   P1.H = _freertos_global_imask;\n\t	\
				   R1 = [ P1 ];		\n\t			\
				   R0 = R0 | R1;	\n\t			\
				   P1 = [ SP ++ ];	\n\t			\
				   R1 = [ SP ++ ];	\n\t			\
				   /* should jump here by offset */ \
				   STI R0;	\n\t"					\
				   ::								\
				   : "R0"							\
				   )	

#define portENTER_CRITICAL()		portDISABLE_INTERRUPTS()
#define portEXIT_CRITICAL()			portENABLE_INTERRUPTS()

/*-----------------------------------------------------------*/

/* Task utilities. */
extern void vContextSwitch(void);	/* interrupt 14 on BFin */
extern void vTickISR(void);
/* FreeRTOS does YIELD() in critical section... 
 * So we have to re-enable soft int 14 here
 */
#define portYIELD() 						\
	asm volatile ("[ -- SP ] = R0;	\n\t	\
					CLI R0;			\n\t	\
					BITSET (R0, 14);\n\t	\
					STI R0;			\n\t	\
					R0 = [ SP ++ ];	\n\t	\
					RAISE 14;")
					
#define vStartFirstTask() portYIELD()

/*-----------------------------------------------------------*/

/* Hardware specifics. */
#define portBYTE_ALIGNMENT			4
#define portSTACK_GROWTH			( -1 )
/* time per tick in ms */
#define portTICK_RATE_MS			( ( portTickType ) 1000 / configTICK_RATE_HZ )		
#define portNOP()					asm volatile ( "NOP;" )
/*-----------------------------------------------------------*/

/* Task function macros as described on the FreeRTOS.org WEB site. */
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )
#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )

#endif /* PORTMACRO_H */

