/** ###################################################################
**     THIS BEAN MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename  : TickTimer.C
**     Project   : RTOSdemo
**     Processor : MC9S12XEP100MAG
**     Beantype  : TimerInt
**     Version   : Bean 02.151, Driver 01.16, CPU db: 3.00.002
**     Compiler  : CodeWarrior HCS12X C Compiler
**     Date/Time : 10.04.08, 10:13
**     Abstract  :
**         This bean "TimerInt" implements a periodic interrupt.
**         When the bean and its events are enabled, the "OnInterrupt"
**         event is called periodically with the period that you specify.
**         TimerInt supports also changing the period in runtime.
**         The source of periodic interrupt can be timer compare or reload
**         register or timer-overflow interrupt (of free running counter).
**     Settings  :
**         Timer name                  : ECT_Counter (16-bit)
**         Compare name                : ECTTC0
**         Counter shared              : No
**
**         High speed mode
**             Prescaler               : divide-by-1
**             Clock                   : 375000 Hz
**           Initial period/frequency
**             Xtal ticks              : 4000
**             microseconds            : 1000
**             milliseconds            : 1
**             seconds (real)          : 0.0010000
**             Hz                      : 1000
**             kHz                     : 1
**
**         Runtime setting             : period/frequency interval (continual setting)
**             ticks                   : 4000 to 400000 ticks
**             microseconds            : 1000 to 100000 microseconds
**             milliseconds            : 1 to 100 milliseconds
**             seconds (real)          : 0.0010000 to 0.1000000 seconds
**             Hz                      : 10 to 1000 Hz
**
**         Initialization:
**              Timer                  : Enabled
**              Events                 : Enabled
**
**         Timer registers
**              Counter                : ECT_TCNT  [$0044]
**              Mode                   : ECT_TIOS  [$0040]
**              Run                    : ECT_TSCR1 [$0046]
**              Prescaler              : ECT_TSCR2 [$004D]
**
**         Compare registers
**              Compare                : ECT_TC0   [$0050]
**
**         Flip-flop registers
**              Mode                   : ECT_TCTL2 [$0049]
**     Contents  :
**         Enable           - byte TickTimer_Enable(void);
**         Disable          - byte TickTimer_Disable(void);
**         SetPeriodTicks16 - byte TickTimer_SetPeriodTicks16(word Ticks);
**         SetPeriodTicks32 - byte TickTimer_SetPeriodTicks32(dword Ticks);
**         SetPeriodUS      - byte TickTimer_SetPeriodUS(word Time);
**         SetPeriodMS      - byte TickTimer_SetPeriodMS(word Time);
**         SetFreqHz        - byte TickTimer_SetFreqHz(word Freq);
**
**     (c) Copyright UNIS, spol. s r.o. 1997-2007
**     UNIS, spol. s r.o.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/

/* MODULE TickTimer. */

#include "Events.h"
#include "TickTimer.h"

#pragma DATA_SEG TickTimer_DATA
#pragma CODE_SEG TickTimer_CODE
#pragma CONST_SEG DEFAULT

static bool EnUser;                    /* Enable/Disable device by user */
static word CmpVal;                    /* Value of the load register */

/* Internal method prototypes */


/* End of Internal methods declarations */

/*
** ===================================================================
**     Method      :  TickTimer_Enable (bean TimerInt)
**
**     Description :
**         This method enables the bean - it starts the timer. Events
**         may be generated (<DisableEvent>/<EnableEvent>).
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte TickTimer_Enable(void)
{
  if (!EnUser) {                       /* Is the device disabled by user? */
    EnUser = TRUE;                     /* If yes then set the flag "device enabled" */
    ECT_TC0 = ECT_TCNT + CmpVal;       /* Store new value to the compare register */ 
    ECT_TFLG1 = 0x01;                  /* Reset interrupt request flag */ 
    ECT_TIE_C0I = 1;                   /* Enable interrupt */ 
    ECT_TSCR1_TEN = 1;                 /* Start timer */ 
  }
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_Disable (bean TimerInt)
**
**     Description :
**         This method disables the bean - it stops the timer. No
**         events will be generated.
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte TickTimer_Disable(void)
{
  EnUser = FALSE;                      /* If yes then set the flag "device disabled" */
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_SetPeriodTicks16 (bean TimerInt)
**
**     Description :
**         This method sets the new period of the generated events.
**         The period is expressed in main oscillator (usualy External
**         osc., internal osc. if exits and external osc. is disabled)
**         ticks as a 16-bit unsigned integer number.
**         This method is available only if the runtime setting type
**         'from interval' is selected in the Timing dialog box in
**         the Runtime setting area.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Ticks           - Period to set [in Xtal ticks]
**                      (4000 to 65535 ticks)
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_MATH - Overflow during evaluation
**                           ERR_RANGE - Parameter out of range
** ===================================================================
*/
byte TickTimer_SetPeriodTicks16(word Ticks)
{
  dlong rtval;                         /* Result of two 32-bit numbers multiplication */
  word rtword;                         /* Result of 64-bit number division */
  
  if (Ticks < 0x0FA0) {                /* Is the given value out of range? */
    return ERR_RANGE;                  /* If yes then error */
  }
  PE_Timer_LngMul((dword)Ticks,(dword)0x18000000LU,&rtval); /* Multiply given value and High speed mode coefficient */
  if (PE_Timer_LngHi4(rtval[0],rtval[1],&rtword)) { /* Is the result greater or equal than 0x00010000 ? */
    rtword = 0xFFFF;                   /* If yes then use maximal possible value */
  }
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  CmpVal = rtword;                     /* Store given value to the variable CmpVal */
  ECT_TC0 = ECT_TCNT + rtword;         /* Store new value to the compare register */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */ 
  ECT_TIE_C0I = 1;                     /* Enable interrupt */ 
  ECT_TSCR1_TEN = 1;                   /* Start timer */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_SetPeriodTicks32 (bean TimerInt)
**
**     Description :
**         This method sets the new period of the generated events.
**         The period is expressed in main oscillator (usualy External
**         osc., internal osc. if exits and external osc. is disabled)
**         ticks as a 32-bit unsigned integer number.
**         This method is available only if the runtime setting type
**         'from interval' is selected in the Timing dialog box in
**         the Runtime setting area.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Ticks           - Period to set [in Xtal ticks]
**                      (4000 to 400000 ticks)
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_MATH - Overflow during evaluation
**                           ERR_RANGE - Parameter out of range
** ===================================================================
*/
byte TickTimer_SetPeriodTicks32(dword Ticks)
{
  dlong rtval;                         /* Result of two 32-bit numbers multiplication */
  word rtword;                         /* Result of 64-bit number division */
  
  if ((Ticks > 0x00061A80) || (Ticks < 0x0FA0)) { /* Is the given value out of range? */
    return ERR_RANGE;                  /* If yes then error */
  }
  PE_Timer_LngMul(Ticks,(dword)0x18000000LU,&rtval); /* Multiply given value and High speed mode coefficient */
  if (PE_Timer_LngHi4(rtval[0],rtval[1],&rtword)) { /* Is the result greater or equal than 0x00010000 ? */
    rtword = 0xFFFF;                   /* If yes then use maximal possible value */
  }
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  CmpVal = rtword;                     /* Store given value to the variable CmpVal */
  ECT_TC0 = ECT_TCNT + rtword;         /* Store new value to the compare register */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */ 
  ECT_TIE_C0I = 1;                     /* Enable interrupt */ 
  ECT_TSCR1_TEN = 1;                   /* Start timer */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_SetPeriodUS (bean TimerInt)
**
**     Description :
**         This method sets the new period of the generated events.
**         The period is expressed in microseconds as a 16-bit
**         unsigned integer number.
**         This method is available only if the runtime setting type
**         'from interval' is selected in the Timing dialog box in
**         the Runtime setting area.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Time            - Period to set [in microseconds]
**                      (1000 to 65535 microseconds)
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_MATH - Overflow during evaluation
**                           ERR_RANGE - Parameter out of range
** ===================================================================
*/
byte TickTimer_SetPeriodUS(word Time)
{
  dlong rtval;                         /* Result of two 32-bit numbers multiplication */
  word rtword;                         /* Result of 64-bit number division */
  
  if (Time < 0x03E8) {                 /* Is the given value out of range? */
    return ERR_RANGE;                  /* If yes then error */
  }
  PE_Timer_LngMul((dword)Time,(dword)0x60000000LU,&rtval); /* Multiply given value and High speed mode coefficient */
  if (PE_Timer_LngHi4(rtval[0],rtval[1],&rtword)) { /* Is the result greater or equal than 0x00010000 ? */
    rtword = 0xFFFF;                   /* If yes then use maximal possible value */
  }
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  CmpVal = rtword;                     /* Store given value to the variable CmpVal */
  ECT_TC0 = ECT_TCNT + rtword;         /* Store new value to the compare register */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */ 
  ECT_TIE_C0I = 1;                     /* Enable interrupt */ 
  ECT_TSCR1_TEN = 1;                   /* Start timer */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_SetPeriodMS (bean TimerInt)
**
**     Description :
**         This method sets the new period of the generated events.
**         The period is expressed in miliseconds as a 16-bit
**         unsigned integer number.
**         This method is available only if the runtime setting type
**         'from interval' is selected in the Timing dialog box in
**         the Runtime setting area.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Time            - Period to set [in miliseconds]
**                      (1 to 100 milliseconds)
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_MATH - Overflow during evaluation
**                           ERR_RANGE - Parameter out of range
** ===================================================================
*/
byte TickTimer_SetPeriodMS(word Time)
{
  dlong rtval;                         /* Result of two 32-bit numbers multiplication */
  word rtword;                         /* Result of 64-bit number division */
  
  if ((Time > 0x64) || (Time < 0x01)) { /* Is the given value out of range? */
    return ERR_RANGE;                  /* If yes then error */
  }
  PE_Timer_LngMul((dword)Time,(dword)0x01770000LU,&rtval); /* Multiply given value and High speed mode coefficient */
  if (PE_Timer_LngHi2(rtval[0],rtval[1],&rtword)) { /* Is the result greater or equal than 0x00010000 ? */
    rtword = 0xFFFF;                   /* If yes then use maximal possible value */
  }
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  CmpVal = rtword;                     /* Store given value to the variable CmpVal */
  ECT_TC0 = ECT_TCNT + rtword;         /* Store new value to the compare register */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */ 
  ECT_TIE_C0I = 1;                     /* Enable interrupt */ 
  ECT_TSCR1_TEN = 1;                   /* Start timer */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_SetFreqHz (bean TimerInt)
**
**     Description :
**         This method sets the new frequency of the generated
**         events. The frequency is expressed in Hz as a 16-bit
**         unsigned integer number.
**         This method is available only if the runtime setting type
**         'from interval' is selected in the Timing dialog box in
**         the Runtime setting area.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Freq            - Frequency to set [in Hz]
**                      (10 to 1000 Hz)
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_MATH - Overflow during evaluation
**                           ERR_RANGE - Parameter out of range
** ===================================================================
*/
byte TickTimer_SetFreqHz(word Freq)
{
  dlong rtval;                         /* Result of two 32-bit numbers multiplication */
  word rtword;                         /* Result of 64-bit number division */
  
  if ((Freq > 0x03E8) || (Freq < 0x0A)) { /* Is the given value out of range? */
    return ERR_RANGE;                  /* If yes then error */
  }
  rtval[1] = 0x05B8D800 / (dword)Freq; /* Divide high speed mode coefficient by the given value */
  rtval[0] = 0;                        /* Convert result to the type dlong */
  if (PE_Timer_LngHi1(rtval[0],rtval[1],&rtword)) { /* Is the result greater or equal than 65536 ? */
    rtword = 0xFFFF;                   /* If yes then use maximal possible value */
  }
  ECT_TSCR1_TEN = 0;                   /* Stop timer */ 
  ECT_TIE_C0I = 0;                     /* Disable interrupt */ 
  CmpVal = rtword;                     /* Store given value to the variable CmpVal */
  ECT_TC0 = ECT_TCNT + rtword;         /* Store new value to the compare register */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */ 
  ECT_TIE_C0I = 1;                     /* Enable interrupt */ 
  ECT_TSCR1_TEN = 1;                   /* Start timer */ 
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  TickTimer_Init (bean TimerInt)
**
**     Description :
**         Initializes the associated peripheral(s) and the beans 
**         internal variables. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void TickTimer_Init(void)
{
  EnUser = TRUE;                       /* Enable device */
  CmpVal = 0x0177;                     /* Store given value to the variable CmpVal */
  /* ECT_TC0: BIT15=0,BIT14=0,BIT13=0,BIT12=0,BIT11=0,BIT10=0,BIT9=0,BIT8=1,BIT7=0,BIT6=1,BIT5=1,BIT4=1,BIT3=0,BIT2=1,BIT1=1,BIT0=1 */
  setReg16(ECT_TC0, 0x0177);           /* Store given value to the compare register */ 
}

/*lint -save -e718 -e746 -e937 -e957 -e18 -e516 -e532 -e14 Disable MISRA rule (71,72,25) checking. */
/*
** ===================================================================
**     Method      :  TickTimer_Interrupt (bean TimerInt)
**
**     Description :
**         The method services the interrupt of the selected peripheral(s)
**         and eventually invokes event(s) of the bean.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#pragma CODE_SEG __NEAR_SEG NON_BANKED
ISR(TickTimer_Interrupt)
{
  ECT_TC0 += CmpVal;                   /* Add value corresponding with periode */
  ECT_TFLG1 = 0x01;                    /* Reset interrupt request flag */
  TickTimer_OnInterrupt();             /* Invoke user event */
}

#pragma CODE_SEG TickTimer_CODE
/*lint -restore */

/* END TickTimer. */

/*
** ###################################################################
**
**     This file was created by UNIS Processor Expert 2.98 [03.98]
**     for the Freescale HCS12X series of microcontrollers.
**
** ###################################################################
*/
