/** ###################################################################
**     THIS BEAN MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename  : COM0.C
**     Project   : RTOSDemo
**     Processor : MC9S12DP256BCPV
**     Beantype  : AsynchroSerial
**     Version   : Bean 02.433, Driver 01.17, CPU db: 2.87.434
**     Compiler  : CodeWarrior HC12 C Compiler
**     Date/Time : 10.04.08, 10:14
**     Abstract  :
**         This bean "AsynchroSerial" implements an asynchronous serial
**         communication. The bean supports different settings of
**         parity, word width, stop-bit and communication speed,
**         user can select interrupt or polling handler.
**         Communication speed can be changed also in runtime.
**         The bean requires one on-chip asynchronous serial channel.
**     Settings  :
**         Serial channel              : SCI0
**
**         Protocol
**             Init baud rate          : 38400baud
**             Width                   : 8 bits
**             Stop bits               : 1
**             Parity                  : none
**             Breaks                  : Disabled
**
**         Registers
**             Input buffer            : SCI0DRL   [207]
**             Output buffer           : SCI0DRL   [207]
**             Control register        : SCI0CR1   [202]
**             Mode register           : SCI0CR2   [203]
**             Baud setting reg.       : SCI0BD    [200]
**             Special register        : SCI0SR1   [204]
**
**         Input interrupt
**             Vector name             : Vsci0
**             Priority                : 1
**
**         Output interrupt
**             Vector name             : Vsci0
**             Priority                : 1
**
**         Used pins:
**         ----------------------------------------------------------
**           Function | On package           |    Name
**         ----------------------------------------------------------
**            Input   |     89               |  PS0_RxD0
**            Output  |     90               |  PS1_TxD0
**         ----------------------------------------------------------
**
**
**         Used baud modes:
**         ----------------------------------------------------------
**           No.     |    Mode ID            |  Baud rate
**         ----------------------------------------------------------
**            0      |  COM0_BM_38400BAUD      |  38400baud
**            1      |  COM0_BM_19200BAUD      |  19200baud
**            2      |  COM0_BM_9600BAUD      |  9600baud
**            3      |  COM0_BM_4800BAUD      |  4800baud
**         ----------------------------------------------------------
**
**     Contents  :
**         SendChar        - byte COM0_SendChar(COM0_TComData Chr);
**         SetBaudRateMode - byte COM0_SetBaudRateMode(byte Mod);
**
**     (c) Copyright UNIS, spol. s r.o. 1997-2007
**     UNIS, spol. s r.o.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/

/* MODULE COM0. */

#pragma MESSAGE DISABLE C4002 /* WARNING C4002: Result not used is ignored */
#pragma MESSAGE DISABLE C4301 /* INFORMATION C4301: Inline expansion done for function call */

#include "COM0.h"
#include "TickTimer.h"
#include "Byte1.h"
#include "IV1.h"

#pragma DATA_SEG COM0_DATA
#pragma CODE_SEG COM0_CODE



#define OVERRUN_ERR      1             /* Overrun error flag bit   */
#define COMMON_ERR       2             /* Common error of RX       */
#define CHAR_IN_RX       4             /* Char is in the RX buffer     */
#define FULL_TX          8             /* Full transmit buffer     */

static byte SerFlag;                   /* Flags for serial communication */
                                       /* Bit 0 - Overrun error */
                                       /* Bit 1 - Common error of RX */
                                       /* Bit 2 - Char in the RX buffer */
                                       /* Bit 3 - Full TX buffer */
static COM0_TComData BufferRead;       /* Input char for SCI commmunication */
static word PrescHigh;
static byte NumMode;                   /* Number of selected baud mode */

/*
** ===================================================================
**     Method      :  COM0_SendChar (bean AsynchroSerial)
**
**     Description :
**         Sends one character to the channel. If the bean is
**         temporarily disabled (Disable method) SendChar method
**         only stores data into an output buffer. In case of a zero
**         output buffer size, only one character can be stored.
**         Enabling the bean (Enable method) starts the transmission
**         of the stored data. This method is available only if the
**         transmitter property is enabled.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Chr             - Character to send
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_TXFULL - Transmitter is full
** ===================================================================
*/
byte COM0_SendChar(COM0_TComData Chr)
{
  if(SerFlag & FULL_TX) {              /* Is any char in the TX buffer */
    return ERR_TXFULL;                 /* If yes then error */
  }
  EnterCritical();                     /* Save the PS register */
  (void) SCI0SR1;                      /* Reset interrupt request flag */
  SCI0DRL = (byte)Chr;                 /* Store char to transmitter register */
  SCI0CR2_SCTIE = 1;                   /* Enable transmit interrupt */
  SerFlag |= FULL_TX;                  /* Set the flag "full TX buffer" */
  ExitCritical();                      /* Restore the PS register */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  COM0_InterruptRx (bean AsynchroSerial)
**
**     Description :
**         The method services the receive interrupt of the selected 
**         peripheral(s) and eventually invokes the bean's event(s).
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define ON_ERROR    1
#define ON_FULL_RX  2
#define ON_RX_CHAR  4
#pragma INLINE
void COM0_InterruptRx(void)
{
  COM0_TComData Data;                  /* Temporary variable for data */

  Data = SCI0DRL;                      /* Read data from the receiver */
  if(SerFlag & CHAR_IN_RX) {           /* Is a character already present in the receive buffer? */
    SerFlag |= OVERRUN_ERR;            /* If yes then set flag OVERRUN_ERR */
  } else {
    BufferRead = Data;
    SerFlag |= CHAR_IN_RX;             /* Set flag "char in RX buffer" */
  }
}

/*
** ===================================================================
**     Method      :  COM0_InterruptTx (bean AsynchroSerial)
**
**     Description :
**         The method services the receive interrupt of the selected 
**         peripheral(s) and eventually invokes the bean's event(s).
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define ON_FREE_TX  1
#define ON_TX_CHAR  2
#pragma INLINE
void COM0_InterruptTx(void)
{
  SerFlag &= ~FULL_TX;                 /* Reset flag "full TX buffer" */
  SCI0CR2_SCTIE = 0;                   /* Disable transmit interrupt */
}

/*
** ===================================================================
**     Method      :  COM0_Interrupt (bean AsynchroSerial)
**
**     Description :
**         The method services the receive interrupt of the selected 
**         peripheral(s) and eventually invokes the bean's event(s).
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#pragma CODE_SEG __NEAR_SEG NON_BANKED
ISR(COM0_Interrupt)
{
  byte StatReg = getReg(SCI0SR1);

  if (StatReg & (SCI0SR1_OR_MASK | SCI0SR1_FE_MASK | SCI0SR1_NF_MASK)) { /* Is any error flag set? */
    SerFlag |= COMMON_ERR;             /* If yes then set an internal flag */
    (void) SCI0DRL;                    /* Dummy read of data register - clear error bits */
    StatReg &= ~SCI0SR1_RDRF_MASK;     /* Clear the receive data flag to discard the errorneous data */
  }
  if (StatReg & SCI0SR1_RDRF_MASK) {   /* Is the receiver interrupt flag set? */
    COM0_InterruptRx();                /* If yes, then invoke the internal service routine. This routine is inlined. */
  }
  if (SCI0CR2_SCTIE) {                 /* Is the transmitter interrupt enabled? */
    if (StatReg & SCI0SR1_TDRE_MASK) { /* Is the transmitter interrupt flag set? */
      COM0_InterruptTx();              /* If yes, then invoke the internal service routine. This routine is inlined. */
    }
  }
}

#pragma CODE_SEG COM0_CODE
/*
** ===================================================================
**     Method      :  COM0_SetBaudRateMode (bean AsynchroSerial)
**
**     Description :
**         This method changes the channel communication speed (baud
**         rate). This method can be used only if the user specifies
**         a list of possible period settings at design time (see
**         <Timing dialog box> - Runtime setting - from a list of
**         values). Each of these settings constitutes a _/mode/_
**         and Processor Expert^[TM] assigns them a _/mode
**         identifier/_. The prescaler and compare values
**         corresponding to each mode are calculated at design time.
**         The user may switch modes at runtime by referring to a
**         mode identifier. No run-time calculations are performed,
**         all the calculations are performed at design time.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Mod             - Timing mode to set
**                           List of timing mode symbols
**                           defined in COM0.h:
**                           0   COM0_BM_38400BAUD     38400baud
**                           1   COM0_BM_19200BAUD     19200baud
**                           2   COM0_BM_9600BAUD      9600baud
**                           3   COM0_BM_4800BAUD      4800baud
**
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte COM0_SetBaudRateMode(byte Mod)
{
  static const word COM0_PrescHigh[4] = {41,81,163,326};

  if(Mod >= 4) {                       /* Is mode in baud mode list */
    return ERR_VALUE;                  /* If no then error */
  }
  NumMode = Mod;                       /* New baud mode */
  PrescHigh = COM0_PrescHigh[Mod];     /* Prescaler in high speed mode */
  SCI0BD = PrescHigh;                  /* Set prescaler bits */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  COM0_Init (bean AsynchroSerial)
**
**     Description :
**         Initializes the associated peripheral(s) and internal 
**         variables of the bean. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void COM0_Init(void)
{
  PrescHigh = 41;                      /* Precaler in high speed mode */
  SerFlag = 0;                         /* Reset flags */
  NumMode = 0;                         /* Number of selected baud mode */
  /* SCI0CR1: LOOPS=0,SCISWAI=1,RSRC=0,M=0,WAKE=0,ILT=0,PE=0,PT=0 */
  setReg8(SCI0CR1, 64);                 
  /* SCI0SR2: ??=0,??=0,??=0,??=0,??=0,BRK13=0,TXDIR=0,RAF=0 */
  setReg8(SCI0SR2, 0);                  
  (void) SCI0SR1;                      /* Reset interrupt request flags */
  /* SCI0CR2: SCTIE=0,TCIE=0,RIE=0,ILIE=0,TE=0,RE=0,RWU=0,SBK=0 */
  SCI0CR2 = 0;                         /* Disable error interrupts */
  SCI0BD = PrescHigh;                  /* Set prescaler bits */
  SCI0CR2 |= (SCI0CR2_TE_MASK | SCI0CR2_RE_MASK | SCI0CR2_RIE_MASK); /* Enable transmitter, Enable receiver, Enable receiver interrupt */
}



/* END COM0. */


/*
** ###################################################################
**
**     This file was created by UNIS Processor Expert 2.98 [03.98]
**     for the Freescale HCS12 series of microcontrollers.
**
** ###################################################################
*/
