/*
	FreeRTOS.org V4.7.0 - Copyright (C) 2003-2007 Richard Barry.
	TriCore port - written by Adriaan Schmidt, Copyright (C) 2008 Siemens AG.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	The TriCore port is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	version 2 as published by the Free Software Foundation.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************
	See http://www.FreeRTOS.org for documentation, latest information, license
	and contact details.  Please ensure to read the configuration and relevant
	port sections of the online documentation.

	Also see http://www.SafeRTOS.com a version that has been certified for use
	in safety critical systems, plus commercial licensing, development and
	support options.
	***************************************************************************
*/

#ifndef PORTMACRO_H
#define PORTMACRO_H

// include Tricore-Intrinsics
#include <machine/intrinsics.h>

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------
 * Port specific definitions.
 *
 * The settings in this file configure FreeRTOS correctly for the
 * given hardware and compiler.
 *
 * These settings should not be altered.
 *-----------------------------------------------------------
 */

/* Configuration */
#define portUSE_TCB_CLEANUP_HOOK 1

/* Type definitions. */
#define portCHAR		char
#define portFLOAT		float
#define portDOUBLE		double
#define portLONG		long
#define portSHORT		int
#define portSTACK_TYPE	unsigned portLONG
#define portBASE_TYPE	long

#if( configUSE_16_BIT_TICKS == 1 )
	typedef unsigned portSHORT portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffff
#else
	typedef unsigned portLONG portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffffff
#endif
/*-----------------------------------------------------------*/

/* Critical section management. */
#define portNO_CRITICAL_NESTING			( ( unsigned portLONG ) 0 )
#define portINITIAL_CRITICAL_NESTING	( ( unsigned portLONG ) 10 )

#define portENTER_CRITICAL() {                  \
	extern unsigned portLONG ulCriticalNesting; \
	portDISABLE_INTERRUPTS();                   \
	ulCriticalNesting++;                        \
}

#define portEXIT_CRITICAL() {                                \
	extern unsigned portLONG ulCriticalNesting;              \
	if( ulCriticalNesting > portNO_CRITICAL_NESTING )        \
	{                                                        \
		ulCriticalNesting--;                                 \
		if( ulCriticalNesting == portNO_CRITICAL_NESTING ) { \
			portENABLE_INTERRUPTS();                         \
		}                                                    \
	}                                                        \
}

#define portDISABLE_INTERRUPTS()  _disable()
#define portENABLE_INTERRUPTS()   _enable()
/*-----------------------------------------------------------*/

/* Architecture specifics. */
#define portSTACK_GROWTH			( -1 )
#define portTICK_RATE_MS			( ( portTickType ) 1000 / configTICK_RATE_HZ )
#define portBYTE_ALIGNMENT			4
#define portNOP()					_nop()
/*-----------------------------------------------------------*/

/* Kernel utilities */
extern void vPortCleanupTCB();

extern void vPortYield( void );
#define portYIELD() {     \
	portENTER_CRITICAL(); \
	vPortYield();         \
	portEXIT_CRITICAL();  \
}

/*-----------------------------------------------------------*/

/* Task function macros as described on the FreeRTOS.org WEB site. */
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) \
		void vFunction( void *pvParameters )

#define portTASK_FUNCTION( vFunction, pvParameters ) \
		void vFunction( void *pvParameters )

#define portIRQ_FUNCTION_PROTO( vFunction ) \
		void __attribute__(( interrupt_handler )) vFunction( void )

#define portIRQ_FUNCTION( vFunction ) \
		void __attribute__(( interrupt_handler )) vFunction( void )

#ifdef __cplusplus
}
#endif

#endif /* PORTMACRO_H */

